@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or
@@ -88,8 +87,7 @@ class ActionConflict (Conflict):
         if not settings:
             for plugin in self.Setting.Plugin.Context.Plugins.values ():
                 if plugin.Enabled:
-                    pluginActions = GetSettings(plugin, displayOnly=True,
-                                                types=self.ActionTypes)
+                    pluginActions = GetSettings(plugin, types=self.ActionTypes)
 
                     if len(setting.Info) and setting.Info[0] and plugin is not setting.Plugin:
                         settings.extend(ExcludeInternal(pluginActions))
@@ -318,7 +316,7 @@ class PluginConflict(Conflict):
             
             elif conflict[0] == 'RequiresFeature':
                 answer, choice = self.AskUser(self.Plugin, conflict)
-                if ret == gtk.RESPONSE_YES:
+                if answer == gtk.RESPONSE_YES:
                     for plg in conflict[2]:
                         if plg.ShortDesc == choice:
                             enableConflicts = plg.EnableConflicts
@@ -467,7 +465,7 @@ class PluginConflict(Conflict):
             widgets.append(cmb)
 
             okMsg = _("Enable these plugins")
-            cancelMsg_("Don't enable %(plugin)s") % msg_dict
+            cancelMsg = _("Don't enable %(plugin)s") % msg_dict
         
         elif conflict[0] == 'RequiresPlugin':
             msg = _("<b>%(plugin)s</b> requires the plugin <b>%(require)s</b>.")
