@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or
@@ -18,10 +17,12 @@
 # Authors: Quinn Storm (quinn@beryl-project.org)
 #          Patrick Niklaus (marex@opencompositing.org)
 #          Guillaume Seguin (guillaume@segu.in)
+#          Christopher Williams (christopherw@verizon.net)
 # Copyright (C) 2007 Quinn Storm
 
 import pygtk
 import gtk
+import gobject
 import gtk.gdk
 
 import compizconfig
@@ -42,12 +43,29 @@ _ = gettext.gettext
 
 CurrentUpdater = None
 
+# Generic Page
+#
+class GenericPage(gobject.GObject):
+    __gsignals__    = {"go-back" : (gobject.SIGNAL_RUN_FIRST,
+                                    gobject.TYPE_NONE,
+                                    [])}
+
+    LeftWidget = None
+    RightWidget = None
+
+    def __init__(self):
+        gobject.GObject.__init__(self)
+
+    def GoBack(self, widget):
+        self.emit('go-back')
+
 # Plugin Page
 #
-class PluginPage:
-    def __init__(self, plugin, main):
+class PluginPage(GenericPage):
+
+    def __init__(self, plugin):
+        GenericPage.__init__(self)
         self.Plugin = plugin
-        self.Main = main
         self.LeftWidget = gtk.VBox(False, 10)
         self.LeftWidget.set_border_width(10)
 
@@ -78,7 +96,7 @@ class PluginPage:
         self.NotFoundBox = None
         
         if plugin.Name != 'core':
-            Tooltips.set_tip(self.FilterEntry, _("Search %s Plugin Options") % plugin.ShortDesc)
+            self.FilterEntry.set_tooltip_text(_("Search %s Plugin Options") % plugin.ShortDesc)
             enableLabel = Label()
             enableLabel.set_markup(HeaderMarkup % (_("Use This Plugin")))
             enableLabel.connect("style-set", self.HeaderStyleSet)
@@ -88,23 +106,24 @@ class PluginPage:
             self.LeftWidget.pack_start(enableCheckCont, False, False)
             enableCheck = gtk.CheckButton()
             enableCheck.add(Label(_("Enable %s") % plugin.ShortDesc, 120))
-            Tooltips.set_tip(enableCheck, plugin.LongDesc)
+            enableCheck.set_tooltip_text(plugin.LongDesc)
             enableCheck.set_active(plugin.Enabled)
             enableCheck.set_sensitive(plugin.Context.AutoSort)
             enableCheckCont.pack_start(enableCheck, True, True)
             enableCheck.connect('toggled', self.EnablePlugin)
         else:
-            Tooltips.set_tip(self.FilterEntry, _("Search Compiz Core Options"))
+            self.FilterEntry.set_tooltip_text(_("Search Compiz Core Options"))
         
         backButton = gtk.Button(gtk.STOCK_GO_BACK)
         backButton.set_use_stock(True)
         self.LeftWidget.pack_end(backButton, False, False)
-        backButton.connect('clicked', main.BackToMain)
+        backButton.connect('clicked', self.GoBack)
         self.RightWidget = gtk.Notebook()
         self.RightWidget.set_scrollable(True)
         self.Pages = []
 
-        for name, group in sorted(plugin.Groups.items()):
+        sortedGroups = sorted(plugin.Groups.items(), key=GroupIndexKeyFunc)
+        for (name, (groupIndex, group)) in sortedGroups:
             name = name or _("General")
             groupPage = GroupPage(name, group)
             groupPage.Wrap()
@@ -199,14 +218,34 @@ class PluginPage:
         else:
             widget.set_active(self.Plugin.Enabled)
         self.Plugin.Context.Write()
-        self.Block = self.Block-1
+        self.Block -= 1
+        GlobalUpdater.UpdatePlugins()
+
+    # Checks if any edit dialog is open, and if so, makes sure a refresh
+    # happens when it closes.
+    def CheckDialogs(self, basePlugin, main):
+        for groupPage in self.Pages:
+            if isinstance(groupPage, GroupPage):
+                for sga in groupPage.subGroupAreas:
+                    for setting in sga.MySettings:
+                        if isinstance(setting, BaseListSetting) and \
+                        setting.EditDialog and setting.EditDialogOpen:
+                            setting.PageToBeRefreshed = (self, basePlugin, main)
+                            return False
+        return True
+
+    def RefreshPage(self, basePlugin, main):
+        curPage = self.RightWidget.get_current_page ()
+        main.BackToMain (None)
+        main.MainPage.ShowPlugin (None, basePlugin)
+        main.CurrentPage.RightWidget.set_current_page (curPage)
 
 # Filter Page
 #
-class FilterPage(object):
-    def __init__(self, main, context):
+class FilterPage(GenericPage):
+    def __init__(self, context):
+        GenericPage.__init__(self)
         self.Context = context
-        self.Main = main
         self.LeftWidget = gtk.VBox(False, 10)
         self.LeftWidget.set_border_width(10)
         self.RightWidget = gtk.Notebook()
@@ -231,7 +270,7 @@ class FilterPage(object):
         else:
             self.FilterEntry = gtk.Entry()
 
-        Tooltips.set_tip(self.FilterEntry, _("Enter a filter.\nClick the keyboard image to grab a key for which to search."))
+        self.FilterEntry.set_tooltip_text(_("Enter a filter.\nClick the keyboard image to grab a key for which to search."))
         self.FilterEntry.connect("changed", self.FilterChanged)
         self.LeftWidget.pack_start(self.FilterEntry, False, False)
 
@@ -260,7 +299,7 @@ class FilterPage(object):
         # Back Button
         self.BackButton = gtk.Button(gtk.STOCK_GO_BACK)
         self.BackButton.set_use_stock(True)
-        self.BackButton.connect('clicked', self.BackToMain)
+        self.BackButton.connect('clicked', self.GoBack)
         self.LeftWidget.pack_end(self.BackButton, False, False)
 
         self.NotFoundBox = None
@@ -330,7 +369,7 @@ class FilterPage(object):
 
         box = gtk.VBox()
         box.set_border_width(5)
-        progress = Popup(parent=main, child=box)
+        progress = Popup(child=box)
         progress.connect("delete-event", lambda *a: True)
         progress.set_title(_("Loading Advanced Search"))
         bar = gtk.ProgressBar()
@@ -353,9 +392,10 @@ class FilterPage(object):
             label.set_markup("<i>%s</i>" %protect_pango_markup(Plugin.ShortDesc))
             gtk_process_events()
 
-            groups = {}
-            for name, group in Plugin.Groups.items():
-                groups[name] = GroupPage(name or _('General'), group)
+            groups = []
+            sortedGroups = sorted(Plugin.Groups.items(), key=GroupIndexKeyFunc)
+            for (name, (groupIndex, group)) in sortedGroups:
+                groups.append((name, GroupPage(name or _('General'), group)))
             self.GroupPages[plugin] = groups
 
         self.Level = FilterName | FilterLongDesc
@@ -381,7 +421,7 @@ class FilterPage(object):
     def Filter(self, text, level=FilterAll):
         text = text.lower()
         for plugin, groups in self.GroupPages.items():
-            results = dict((n, sg) for (n, sg) in groups.items() if sg.Filter(text, level=level))
+            results = dict((n, sg) for (n, sg) in groups if sg.Filter(text, level=level))
             if results:
                 yield plugin, results
 
@@ -562,7 +602,7 @@ class FilterPage(object):
             self.PackedGroups = []
             for plugin in plugins:
                 box = gtk.VBox()
-                for page in self.GroupPages[plugin.Name].values():
+                for (pageName, page) in self.GroupPages[plugin.Name]:
                     box.pack_start(page.Label, False, False)
                     box.pack_start(page.Widget, False, False)
 
@@ -607,21 +647,19 @@ class FilterPage(object):
         elif self.NotFoundBox:
             self.HideFilterError()
 
-    def BackToMain(self, widget):
-        self.RightChild.destroy()
+    def GoBack(self, widget):
         for groups in self.GroupPages.values():
-            for page in groups.values():
+            for (pageName, page) in groups:
                 page.SetContainer.destroy()
         self.GroupPages = None
 
-        self.Main.BackToMain(widget)
+        self.emit('go-back')
 
 # Profile and Backend Page
 #
 class ProfileBackendPage(object):
-    def __init__(self, main, context):
+    def __init__(self, context):
         self.Context = context
-        self.Main = main
         rightChild = gtk.VBox()
         rightChild.set_border_width(10)
 
@@ -629,10 +667,10 @@ class ProfileBackendPage(object):
         profileBox = gtk.HBox()
         profileBox.set_spacing(5)
         profileAdd = gtk.Button()
-        Tooltips.set_tip(profileAdd, _("Add a New Profile"))
+        profileAdd.set_tooltip_text(_("Add a New Profile"))
         profileAdd.set_image(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON))
         self.ProfileRemoveButton = profileRemove = gtk.Button()
-        Tooltips.set_tip(profileRemove, _("Remove This Profile"))
+        profileRemove.set_tooltip_text(_("Remove This Profile"))
         profileRemove.set_image(gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_BUTTON))
         self.ProfileComboBox = gtk.combo_box_new_text()
         self.ProfileComboBox.set_sensitive(self.Context.CurrentBackend.ProfileSupport)
@@ -658,13 +696,13 @@ class ProfileBackendPage(object):
         self.ProfileImportExportBox = gtk.HBox()
         self.ProfileImportExportBox.set_spacing(5)
         profileImportButton = gtk.Button(_("Import"))
-        Tooltips.set_tip(profileImportButton, _("Import a CompizConfig Profile"))
+        profileImportButton.set_tooltip_text(_("Import a CompizConfig Profile"))
         profileImportAsButton = gtk.Button(_("Import as..."))
-        Tooltips.set_tip(profileImportAsButton, _("Import a CompizConfig Profile as a new profile"))
+        profileImportAsButton.set_tooltip_text(_("Import a CompizConfig Profile as a new profile"))
         profileExportButton = gtk.Button(_("Export"))
-        Tooltips.set_tip(profileExportButton, _("Export your CompizConfig Profile"))
+        profileExportButton.set_tooltip_text(_("Export your CompizConfig Profile"))
         profileResetButton = gtk.Button(_("Reset to defaults"))
-        Tooltips.set_tip(profileResetButton, _("Reset your CompizConfig Profile to the global defaults"))
+        profileResetButton.set_tooltip_text(_("Reset your CompizConfig Profile to the global defaults"))
         profileResetButton.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_BUTTON))
         profileImportButton.set_image(gtk.image_new_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_BUTTON))
         profileImportAsButton.set_image(gtk.image_new_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_BUTTON))
@@ -776,19 +814,20 @@ class ProfileBackendPage(object):
     def ResetProfile(self, widget):
         
         for plugin in self.Context.Plugins.values():
-            settings = sum((v.values() for v in [plugin.Display]+[plugin.Screens[CurrentScreenNum]]), [])
+            settings = GetSettings(plugin)
             for setting in settings:
                 setting.Reset()
 
-        activePlugins = self.Context.Plugins['core'].Display['active_plugins'].Value
+        activePlugins = self.Context.Plugins['core'].Screen['active_plugins'].Value
         for plugin in self.Context.Plugins.values():
             plugin.Enabled = plugin.Name in activePlugins
         self.Context.Write()
         GlobalUpdater.UpdatePlugins()
     
     def ExportProfile(self, widget):
+        main = widget.get_toplevel()
         b = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK)
-        chooser = gtk.FileChooserDialog(title=_("Save file.."), parent=self.Main, buttons=b, action=gtk.FILE_CHOOSER_ACTION_SAVE)
+        chooser = gtk.FileChooserDialog(title=_("Save file.."), parent=main, buttons=b, action=gtk.FILE_CHOOSER_ACTION_SAVE)
         chooser.set_current_folder(os.environ.get("HOME"))
         self.CreateFilter(chooser)
         ret = chooser.run()
@@ -800,13 +839,15 @@ class ProfileBackendPage(object):
             dlg.set_markup(_("Do you want to skip default option values while exporting your profile?"))
             ret = dlg.run()
             dlg.destroy()
+            if not path.endswith(".profile"):
+                path = "%s.profile" % path
             self.Context.Export(path, ret == gtk.RESPONSE_YES)
 
-    def ImportProfileDialog (self):
+    def ImportProfileDialog (self, main):
         b = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
              gtk.STOCK_OPEN, gtk.RESPONSE_OK)
         chooser = gtk.FileChooserDialog (title = _("Open file.."),
-                                         parent = self.Main, buttons = b)
+                                         parent = main, buttons = b)
         chooser.set_current_folder (os.environ.get ("HOME"))
         self.CreateFilter (chooser)
         ret = chooser.run ()
@@ -817,8 +858,8 @@ class ProfileBackendPage(object):
             return path
         return None
 
-    def ProfileNameDialog (self):
-        dlg = gtk.Dialog (_("Enter a profile name"), self.Main,
+    def ProfileNameDialog (self, main):
+        dlg = gtk.Dialog (_("Enter a profile name"), main,
                           gtk.DIALOG_MODAL)
         dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
         dlg.add_button (gtk.STOCK_ADD, gtk.RESPONSE_OK)
@@ -838,16 +879,18 @@ class ProfileBackendPage(object):
         return None
 
     def ImportProfile (self, widget):
-        path = self.ImportProfileDialog ()
+        main = widget.get_toplevel ()
+        path = self.ImportProfileDialog (main)
         if path:
             self.Context.Import (path)
         GlobalUpdater.UpdatePlugins()
 
     def ImportProfileAs (self, widget):
-        path = self.ImportProfileDialog ()
+        main = widget.get_toplevel ()
+        path = self.ImportProfileDialog (main)
         if not path:
             return
-        name = self.ProfileNameDialog ()
+        name = self.ProfileNameDialog (main)
         if not name:
             return
         self.Context.CurrentProfile = ccs.Profile (self.Context, name)
@@ -855,7 +898,8 @@ class ProfileBackendPage(object):
         self.Context.Import (path)
 
     def AddProfile (self, widget):
-        name = self.ProfileNameDialog ()
+        main = widget.get_toplevel ()
+        name = self.ProfileNameDialog (main)
         if name:
             self.Context.CurrentProfile = ccs.Profile (self.Context, name)
             self.UpdateProfiles (name)
@@ -893,9 +937,8 @@ class ProfileBackendPage(object):
 # Plugin List Page
 #
 class PluginListPage(object):
-    def __init__(self, main, context):
+    def __init__(self, context):
         self.Context = context
-        self.Main = main
         self.Block = 0
         rightChild = gtk.VBox()
         rightChild.set_border_width(10)
@@ -1003,7 +1046,7 @@ class PluginListPage(object):
         GlobalUpdater.UpdatePlugins()
 
     def UpdateEnabledPluginsList(self):
-        activePlugins = self.Context.Plugins['core'].Display['active_plugins'].Value
+        activePlugins = self.Context.Plugins['core'].Screen['active_plugins'].Value
         
         self.EnabledPluginsList.clear()
 
@@ -1011,7 +1054,7 @@ class PluginListPage(object):
             self.EnabledPluginsList.append(name)
 
     def UpdateDisabledPluginsList(self):
-        activePlugins = self.Context.Plugins['core'].Display['active_plugins'].Value
+        activePlugins = self.Context.Plugins['core'].Screen['active_plugins'].Value
 
         self.DisabledPluginsList.clear()
 
@@ -1025,12 +1068,9 @@ class PluginListPage(object):
         dlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK).grab_default()
         dlg.set_default_response(gtk.RESPONSE_OK)
         
-        ebox = gtk.EventBox()
         label = gtk.Label(_("Plugin name:"))
-        ebox.add(label)
-        
-        Tooltips.set_tip(ebox, _("Insert plugin name"))
-        dlg.vbox.pack_start(ebox)
+        label.set_tooltip_text(_("Insert plugin name"))
+        dlg.vbox.pack_start(label)
         
         entry = gtk.Entry()
         entry.props.activates_default = True
@@ -1059,17 +1099,17 @@ class PluginListPage(object):
         self.Block += 1
         plugins = self.EnabledPluginsList.get_list()
 
-        self.Context.Plugins['core'].Display['active_plugins'].Value = plugins
+        self.Context.Plugins['core'].Screen['active_plugins'].Value = plugins
         self.Context.Write()
         self.UpdateDisabledPluginsList()
         self.Block -= 1
 
 # Preferences Page
 #
-class PreferencesPage(object):
-    def __init__(self, main, context):
+class PreferencesPage(GenericPage):
+    def __init__(self, context):
+        GenericPage.__init__(self)
         self.Context = context
-        self.Main = main
         self.LeftWidget = gtk.VBox(False, 10)
         self.LeftWidget.set_border_width(10)
         self.RightWidget = gtk.Notebook()
@@ -1099,7 +1139,7 @@ class PreferencesPage(object):
         aboutFrame.pack_start(aboutImage, False, False)
         aboutFrame.pack_start(Label(_("About CCSM...")), False, False)
         aboutButton.add(aboutFrame)
-        Tooltips.set_tip(aboutButton, _("About"))
+        aboutButton.set_tooltip_text(_("About"))
         aboutButton.connect('clicked', self.ShowAboutDialog)
         aboutBin = gtk.HBox()
         aboutBin.set_border_width(10)
@@ -1110,15 +1150,15 @@ class PreferencesPage(object):
         # Back Button
         backButton = gtk.Button(gtk.STOCK_GO_BACK)
         backButton.set_use_stock(True)
-        backButton.connect('clicked', self.Main.BackToMain)
+        backButton.connect('clicked', self.GoBack)
         self.LeftWidget.pack_end(backButton, False, False)
 
         # Profile & Backend Page
-        self.ProfileBackendPage = ProfileBackendPage(main, context)
+        self.ProfileBackendPage = ProfileBackendPage(context)
         self.RightWidget.append_page(self.ProfileBackendPage.Widget, gtk.Label(_("Profile & Backend")))
 
         # Plugin List
-        self.PluginListPage = PluginListPage(main, context)
+        self.PluginListPage = PluginListPage(context)
         self.RightWidget.append_page(self.PluginListPage.Widget, gtk.Label(_("Plugin List")))
 
     StyleBlock = 0
@@ -1137,6 +1177,157 @@ class PreferencesPage(object):
         about.run()
         about.destroy()
 
+# Main Page
+#
+class MainPage(object):
+    def __init__(self, main, context):
+        self.Context = context
+        self.Main    = main
+        sidebar = gtk.VBox(False, 10)
+        sidebar.set_border_width(10)
+        pluginWindow = PluginWindow(self.Context)
+        pluginWindow.connect('show-plugin', self.ShowPlugin)
+
+        # Filter
+        filterLabel = Label()
+        filterLabel.set_markup(HeaderMarkup % (_("Filter")))
+        filterLabel.connect("style-set", self.HeaderStyleSet)
+        filterLabel.props.xalign = 0.1
+        if has_sexy:
+            filterEntry = sexy.IconEntry()
+            filterEntry.add_clear_button()
+        else:
+            filterEntry = gtk.Entry()
+        filterEntry.set_tooltip_text(_("Filter your Plugin list"))
+        filterEntry.connect("changed", self.FilterChanged)
+        self.filterEntry = filterEntry
+
+        # Screens
+        if len(getScreens()) > 1:
+            screenBox = gtk.combo_box_new_text()
+            for screen in getScreens():
+                screenBox.append_text(_("Screen %i") % screen)
+            name = self.Context.CurrentBackend.Name
+            screenBox.set_active(CurrentScreenNum)
+            screenBox.connect("changed", self.ScreenChanged)
+            screenLabel = Label()
+            screenLabel.set_markup(HeaderMarkup % (_("Screen")))
+            screenLabel.connect("style-set", self.HeaderStyleSet)
+
+            sidebar.pack_start(screenLabel, False, False)
+            sidebar.pack_start(screenBox, False, False)
+
+        # Categories
+        categoryBox = gtk.VBox()
+        categoryBox.set_border_width(5)
+        categories = ['All'] + sorted(pluginWindow.get_categories(), key=CategoryKeyFunc)
+        for category in categories:
+            # name: untranslated name/interal identifier
+            # label: translated name
+            name = category or 'Uncategorized'
+            label = _(name)
+            iconName = name.lower ().replace (" ", "_")
+            categoryToggleIcon = Image (name = iconName, type = ImageCategory,
+                                        size = 22)
+            categoryToggleLabel = Label (label)
+            align = gtk.Alignment (0, 0.5, 1, 1)
+            align.set_padding (0, 0, 0, 10)
+            align.add (categoryToggleIcon)
+            categoryToggleBox = gtk.HBox ()
+            categoryToggleBox.pack_start (align, False, False)
+            categoryToggleBox.pack_start (categoryToggleLabel, True, True)
+            categoryToggle = PrettyButton ()
+            categoryToggle.add(categoryToggleBox)
+            categoryToggle.connect("clicked", self.ToggleCategory, category)
+            categoryBox.pack_start(categoryToggle, False, False)
+        categoryLabel = Label()
+        categoryLabel.props.xalign = 0.1
+        categoryLabel.set_markup(HeaderMarkup % (_("Category")))
+        categoryLabel.connect("style-set", self.HeaderStyleSet)
+
+        # Exit Button
+        exitButton = gtk.Button(gtk.STOCK_CLOSE)
+        exitButton.set_use_stock(True)
+        exitButton.connect('clicked', self.Main.Quit)
+
+        # Advanced Search
+        searchLabel = Label()
+        searchLabel.set_markup(HeaderMarkup % (_("Advanced Search")))
+        searchLabel.connect("style-set", self.HeaderStyleSet)
+        searchImage = gtk.Image()
+        searchImage.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_BUTTON)
+        searchButton = PrettyButton()
+        searchButton.connect("clicked", self.ShowAdvancedFilter)
+        searchButton.set_relief(gtk.RELIEF_NONE)
+        searchFrame = gtk.HBox()
+        searchFrame.pack_start(searchLabel, False, False)
+        searchFrame.pack_end(searchImage, False, False)
+        searchButton.add(searchFrame)
+
+        # Preferences
+        prefLabel = Label()
+        prefLabel.set_markup(HeaderMarkup % (_("Preferences")))
+        prefLabel.connect("style-set", self.HeaderStyleSet)
+        prefImage = gtk.Image()
+        prefImage.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_BUTTON)
+        prefButton = PrettyButton()
+        prefButton.connect("clicked", self.ShowPreferences)
+        prefButton.set_relief(gtk.RELIEF_NONE)
+        prefFrame = gtk.HBox()
+        prefFrame.pack_start(prefLabel, False, False)
+        prefFrame.pack_end(prefImage, False, False)
+        prefButton.add(prefFrame)
+
+        # Pack widgets into sidebar
+        sidebar.pack_start(filterLabel, False, False)
+        sidebar.pack_start(filterEntry, False, False)
+        sidebar.pack_start(categoryLabel, False, False)
+        sidebar.pack_start(categoryBox, False, False)
+        sidebar.pack_end(exitButton, False, False)
+        sidebar.pack_end(searchButton, False, False)
+        sidebar.pack_end(prefButton, False, False)
+
+        self.LeftWidget = sidebar
+        self.RightWidget = pluginWindow
+
+    StyleBlock = 0
+
+    def HeaderStyleSet(self, widget, previous):
+        if self.StyleBlock > 0:
+            return
+        self.StyleBlock += 1
+        for state in (gtk.STATE_NORMAL, gtk.STATE_PRELIGHT, gtk.STATE_ACTIVE):
+            widget.modify_fg(state, widget.style.bg[gtk.STATE_SELECTED])
+        self.StyleBlock -= 1
+
+    def ShowPlugin(self, widget, plugin):
+        pluginPage = PluginPage(plugin)
+        self.Main.SetPage(pluginPage)
+
+    def ShowAdvancedFilter(self, widget):
+        filterPage = FilterPage(self.Context)
+        self.Main.SetPage(filterPage)
+
+    def ShowPreferences(self, widget):
+        preferencesPage = PreferencesPage(self.Context)
+        self.Main.SetPage(preferencesPage)
+
+    def ToggleCategory(self, widget, category):
+        if category == 'All':
+            category = None
+        else:
+            category = category.lower()
+        self.RightWidget.filter_boxes(category, level=FilterCategory)
+
+    def FilterChanged(self, widget):
+        text = widget.get_text().lower()
+        self.RightWidget.filter_boxes(text)
+
+    def ScreenChanged(self, widget):
+        self.Context.Write()
+        self.CurrentScreenNum = widget.get_active()
+        self.Context.Read()
+
 # Page
 #
 class Page(object):
@@ -1145,18 +1336,18 @@ class Page(object):
 
         self.Widget = gtk.EventBox()
         self.Widget.add(self.SetContainer)
-        
+
         self.Empty = True
 
     def Wrap(self):
         scroll = gtk.ScrolledWindow()
         scroll.props.hscrollbar_policy = gtk.POLICY_NEVER
         scroll.props.vscrollbar_policy = gtk.POLICY_AUTOMATIC
-        
+
         view = gtk.Viewport()
         view.set_border_width(5)
         view.set_shadow_type(gtk.SHADOW_NONE)
-        
+
         scroll.add(view)
         view.add(self.Widget)
 
@@ -1177,15 +1368,16 @@ class GroupPage(Page):
         label.set_use_markup(True)
         self.Label.add(label)
         if '' in group:
-            sga = SubGroupArea('', group[''])
+            sga = SubGroupArea('', group[''][1])
             if not sga.Empty:
                 self.SetContainer.pack_start(sga.Widget, False, False)
                 self.Empty = False
                 self.subGroupAreas.append(sga)
 
-        for subGroup in sorted(group):
-            if not subGroup == '':
-                sga = SubGroupArea(subGroup, group[subGroup])
+        sortedSubGroups = sorted(group.items(), key=GroupIndexKeyFunc)
+        for (subGroupName, (subGroupIndex, subGroup)) in sortedSubGroups:
+            if not subGroupName == '':
+                sga = SubGroupArea(subGroupName, subGroup)
                 if not sga.Empty:
                     self.SetContainer.pack_start(sga.Widget, False, False)
                     self.Empty = False
